from __future__ import print_function

import numpy as np
from scipy import signal

# Return a spectrogram image for the current stations vertical trace
# Optionals: [winLen (float),interLen (float),normType (one of sqrt,log,none)]
def spectrogramVert(stream,curTraceSta,**kwargs):
    # Ensure that the vertical trace exists...
    if curTraceSta+'Z' not in [tr.stats.network+'.'+tr.stats.station+'.'+
                               tr.stats.location+tr.stats.channel[-1] for tr in stream]:
        return '$pass'
    # If winLen (window over which a single FFT computed) or interLen (sampling interval in time of spectrogram)...
    # ...does not exist, add in some defaults
    if 'winLen' not in kwargs.keys():
        kwargs['winLen']=1.0
    if 'interLen' not in kwargs.keys():
        kwargs['interLen']=0.5
    if kwargs['interLen']>kwargs['winLen']:
        kwargs['interLen']=0.5*kwargs['winLen']
    # Give default sqrt normalization if none are given, or
    if 'normType' not in kwargs.keys():
        kwargs['normType']='sqrt'
    elif kwargs['normType'] not in ['sqrt','log','none']:
        print('Accepted normalization "normType" args are [sqrt, log, none], assigning sqrt')
        kwargs['normType']='sqrt'
    # Grab the trace and calculate the spectrogram
    net,sta,loc=curTraceSta.split('.')
    trace=stream.select(component='Z',station=sta,network=net,location=loc).merge()[0]
    trace.detrend('linear')
    trace.taper(0.1,type='hann',max_length=2.0)
    sps=trace.stats.sampling_rate
    f, t, psd=signal.spectrogram(trace.data, sps,
                                 nperseg=int(sps*kwargs['winLen']),
                                 noverlap=int(sps*(kwargs['winLen']-kwargs['interLen'])))
    # Ensure the spectrogram is large enough to mean something
    if len(t)<2 or len(f)<2:
        return '$pass'
    # Normalize the psd
    if kwargs['normType']=='sqrt':
        psd=np.sqrt(psd)
    elif kwargs['normType']=='log':
        psd=np.log10(psd)
    # Scale between 0 and 1
    psd=(psd-np.min(psd))/(np.max(psd)-np.min(psd))
    return {'data':psd,'t0':trace.stats.starttime.timestamp,'tDelta':t[1]-t[0],
            'y0':f[0],'yDelta':f[1]-f[0],'label':tr.stats.network+'.'+tr.stats.station+'.'+tr.stats.location,
            'cmapRGBA':np.array([[0,0,0,255],[255,0,0,255]])}


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'Configuration.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ConfDialog(object):
    def setupUi(self, ConfDialog):
        ConfDialog.setObjectName("ConfDialog")
        ConfDialog.resize(695, 449)
        self.horizontalLayout = QtWidgets.QHBoxLayout(ConfDialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.confPrefLayout = QtWidgets.QVBoxLayout()
        self.confPrefLayout.setSpacing(1)
        self.confPrefLayout.setObjectName("confPrefLayout")
        self.confPrefLabel = QtWidgets.QLabel(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPrefLabel.sizePolicy().hasHeightForWidth())
        self.confPrefLabel.setSizePolicy(sizePolicy)
        self.confPrefLabel.setMinimumSize(QtCore.QSize(0, 25))
        self.confPrefLabel.setMaximumSize(QtCore.QSize(16777215, 25))
        self.confPrefLabel.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.confPrefLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.confPrefLabel.setObjectName("confPrefLabel")
        self.confPrefLayout.addWidget(self.confPrefLabel)
        self.confPrefList = MixListWidget(ConfDialog)
        self.confPrefList.setDragDropMode(QtWidgets.QAbstractItemView.NoDragDrop)
        self.confPrefList.setObjectName("confPrefList")
        self.confPrefLayout.addWidget(self.confPrefList)
        self.horizontalLayout.addLayout(self.confPrefLayout)
        self.confPassiveLayout = QtWidgets.QVBoxLayout()
        self.confPassiveLayout.setSpacing(1)
        self.confPassiveLayout.setObjectName("confPassiveLayout")
        self.confPassiveTopLayout = QtWidgets.QHBoxLayout()
        self.confPassiveTopLayout.setSpacing(0)
        self.confPassiveTopLayout.setObjectName("confPassiveTopLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confPassiveTopLayout.addItem(spacerItem)
        self.confPassiveLabel = QtWidgets.QLabel(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPassiveLabel.sizePolicy().hasHeightForWidth())
        self.confPassiveLabel.setSizePolicy(sizePolicy)
        self.confPassiveLabel.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.confPassiveLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.confPassiveLabel.setObjectName("confPassiveLabel")
        self.confPassiveTopLayout.addWidget(self.confPassiveLabel)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confPassiveTopLayout.addItem(spacerItem1)
        self.confPassiveAddButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPassiveAddButton.sizePolicy().hasHeightForWidth())
        self.confPassiveAddButton.setSizePolicy(sizePolicy)
        self.confPassiveAddButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confPassiveAddButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confPassiveAddButton.setAutoDefault(False)
        self.confPassiveAddButton.setDefault(False)
        self.confPassiveAddButton.setFlat(False)
        self.confPassiveAddButton.setObjectName("confPassiveAddButton")
        self.confPassiveTopLayout.addWidget(self.confPassiveAddButton)
        self.confPassiveDelButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPassiveDelButton.sizePolicy().hasHeightForWidth())
        self.confPassiveDelButton.setSizePolicy(sizePolicy)
        self.confPassiveDelButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confPassiveDelButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confPassiveDelButton.setAutoDefault(False)
        self.confPassiveDelButton.setObjectName("confPassiveDelButton")
        self.confPassiveTopLayout.addWidget(self.confPassiveDelButton)
        self.confPassiveLayout.addLayout(self.confPassiveTopLayout)
        self.confPassiveList = MixListWidget(ConfDialog)
        self.confPassiveList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.confPassiveList.setObjectName("confPassiveList")
        self.confPassiveLayout.addWidget(self.confPassiveList)
        self.horizontalLayout.addLayout(self.confPassiveLayout)
        self.confActiveLayout = QtWidgets.QVBoxLayout()
        self.confActiveLayout.setSpacing(1)
        self.confActiveLayout.setObjectName("confActiveLayout")
        self.confActiveTopLayout = QtWidgets.QHBoxLayout()
        self.confActiveTopLayout.setSpacing(0)
        self.confActiveTopLayout.setObjectName("confActiveTopLayout")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confActiveTopLayout.addItem(spacerItem2)
        self.confActiveLabel = QtWidgets.QLabel(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confActiveLabel.sizePolicy().hasHeightForWidth())
        self.confActiveLabel.setSizePolicy(sizePolicy)
        self.confActiveLabel.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.confActiveLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.confActiveLabel.setObjectName("confActiveLabel")
        self.confActiveTopLayout.addWidget(self.confActiveLabel)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confActiveTopLayout.addItem(spacerItem3)
        self.confActiveAddButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confActiveAddButton.sizePolicy().hasHeightForWidth())
        self.confActiveAddButton.setSizePolicy(sizePolicy)
        self.confActiveAddButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confActiveAddButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confActiveAddButton.setAutoDefault(False)
        self.confActiveAddButton.setObjectName("confActiveAddButton")
        self.confActiveTopLayout.addWidget(self.confActiveAddButton)
        self.confActiveDelButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confActiveDelButton.sizePolicy().hasHeightForWidth())
        self.confActiveDelButton.setSizePolicy(sizePolicy)
        self.confActiveDelButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confActiveDelButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confActiveDelButton.setAutoDefault(False)
        self.confActiveDelButton.setObjectName("confActiveDelButton")
        self.confActiveTopLayout.addWidget(self.confActiveDelButton)
        self.confActiveLayout.addLayout(self.confActiveTopLayout)
        self.confActiveList = MixListWidget(ConfDialog)
        self.confActiveList.setObjectName("confActiveList")
        self.confActiveLayout.addWidget(self.confActiveList)
        self.horizontalLayout.addLayout(self.confActiveLayout)

        self.retranslateUi(ConfDialog)
        QtCore.QMetaObject.connectSlotsByName(ConfDialog)

    def retranslateUi(self, ConfDialog):
        _translate = QtCore.QCoreApplication.translate
        ConfDialog.setWindowTitle(_translate("ConfDialog", "Configuration"))
        self.confPrefLabel.setText(_translate("ConfDialog", "Preferences"))
        self.confPrefList.setSortingEnabled(True)
        self.confPassiveLabel.setText(_translate("ConfDialog", "Passive Actions Ordering"))
        self.confPassiveAddButton.setText(_translate("ConfDialog", "+"))
        self.confPassiveDelButton.setText(_translate("ConfDialog", "-"))
        self.confActiveLabel.setText(_translate("ConfDialog", "Active Actions"))
        self.confActiveAddButton.setText(_translate("ConfDialog", "+"))
        self.confActiveDelButton.setText(_translate("ConfDialog", "-"))
        self.confActiveList.setSortingEnabled(True)

from CustomWidgets import MixListWidget



# Author: Andrew.M.G.Reynen
from __future__ import print_function, division

import numpy as np
from PyQt5 import QtWidgets, QtGui, QtCore
from PyQt5.QtCore import Qt
    
# Widget which return key presses, however double click replaces backspace
class MixListWidget(QtWidgets.QListWidget):       
    # Return signal if key was pressed while in focus
    keyPressedSignal=QtCore.pyqtSignal()  
    leaveSignal=QtCore.pyqtSignal()
    
    def __init__(self, parent=None):
        super(MixListWidget, self).__init__(parent)
    
    # Update this widgets last pressed key, and return a signal
    def keyPressEvent(self, ev):
        self.key=ev.key()
        if self.key not in [Qt.Key_Insert,Qt.Key_Delete]:
            return
        self.keyPressedSignal.emit()
        
    # Deselect all items if in a blank space
    def mousePressEvent(self, ev):
        if self.itemAt(ev.pos()) is None:
            self.clearSelection()
        QtWidgets.QListWidget.mousePressEvent(self, ev)
        
    # Swapping a double click with the backspace button...
    # ...wanted to transmit double click as a key
    def mouseDoubleClickEvent(self,ev):
        # Only allow left-double clicks
        if ev.button()!=1:
            return
        self.key=Qt.Key_Backspace
        self.keyPressedSignal.emit()

    # Ensure that key presses are sent to the widget which the mouse is hovering over
    def enterEvent(self,ev):
        super(MixListWidget, self).enterEvent(ev)
        self.setFocus()

    # If ever the list if left by the mouse, emit (used to trigger ordering of passive functions)
    def leaveEvent(self,ev):
        self.leaveSignal.emit()
        
    # Return the lists entries in the order which it appears
    def visualListOrder(self):
        txt=np.array([self.item(i).text() for i in range(self.count())])
        args=np.array([self.indexFromItem(self.item(i)).row() for i in range(self.count())])
        return list(txt[np.argsort(args)])
        
# Generic widget for QListWidget with remove only keyPresses...
# ...currently only used for key bind of delete (if want more keys, must check appropriate lists)
class KeyListWidget(QtWidgets.QListWidget):       
    # Return signal if key was pressed while in focus
    keyPressedSignal = QtCore.pyqtSignal()  
    
    def __init__(self, parent=None):
        super(KeyListWidget, self).__init__(parent)
    
    # Update this widgets last pressed key, and return a signal
    def keyPressEvent(self, ev):
        self.key=ev.key()
        if self.key not in [Qt.Key_Delete]:
            return
        self.keyPressedSignal.emit()
        
    # Deselect all items if in a blank space
    def mousePressEvent(self, ev):
        if self.itemAt(ev.pos()) is None:
            self.clearSelection()
        QtWidgets.QListWidget.mousePressEvent(self, ev)
    
    # Ensure that key presses are sent to the widget which the mouse is hovering over
    def enterEvent(self,ev):
        super(KeyListWidget, self).enterEvent(ev)
        self.setFocus()

    # Return a lists entries in the order which is appears
    def visualListOrder(self):
        txt=np.array([self.item(i).text() for i in range(self.count())])
        args=np.array([self.indexFromItem(self.item(i)).row() for i in range(self.count())])
        return list(txt[np.argsort(args)])

# Convert any key press signal to a human readable string (ignores modifiers like shift, ctrl)
def keyPressToString(ev):
    MOD_MASK = (Qt.CTRL | Qt.ALT | Qt.SHIFT | Qt.META)
    keyname = None
    key = ev.key()
    modifiers = int(ev.modifiers())
    if key in [Qt.Key_Shift,Qt.Key_Alt,Qt.Key_Control,Qt.Key_Meta]:
        pass
    elif (modifiers and modifiers & MOD_MASK==modifiers and key>0):
        keyname=QtGui.QKeySequence(modifiers+key).toString()
    else:
        keyname=QtGui.QKeySequence(key).toString()
    return keyname

# Line edit which returns key-bind strings
class KeyBindLineEdit(QtWidgets.QLineEdit):
    keyPressed=QtCore.pyqtSignal(str)
    
    def __init__(self, parent=None):
        super(KeyBindLineEdit, self).__init__(parent)    
        self.MOD_MASK = (Qt.CTRL | Qt.ALT | Qt.SHIFT | Qt.META)
    
    # If any usual key bind was pressed, return the human recognizable string
    def keyPressEvent(self, ev):
        keyname = keyPressToString(ev)
        if keyname==None:
            return
        self.keyPressed.emit(keyname)

# Line edit which returns signal upon being double cliced     
class DblClickLineEdit(QtWidgets.QLineEdit):
    doubleClicked=QtCore.pyqtSignal()  
    
    def __init__(self, parent=None):
        super(DblClickLineEdit, self).__init__(parent)   
    
    def mouseDoubleClickEvent(self,ev):
        self.doubleClicked.emit()
        
# Line edit which returns a signal upon being hovered out of
class HoverLineEdit(QtWidgets.QLineEdit):
    hoverOut = QtCore.pyqtSignal()
    
    def __init__(self, parent=None):
        super(HoverLineEdit, self).__init__(parent)   
        self.changeState=False        
        self.textChanged.connect(self.updateChangeState)
        
    def leaveEvent(self, ev):
        # Only emit if the text was changed
        if self.changeState:
            self.hoverOut.emit()
            self.changeState=False
    
    def updateChangeState(self,ev):
        self.changeState=True
        
# Generic widget for QListWidget with remove only keyPresses
class DblClickLabelWidget(QtWidgets.QLabel):       
    # Return signal if label was double clicked
    doubleClicked=QtCore.pyqtSignal()  
    
    def __init__(self, parent=None):
        super(DblClickLabelWidget, self).__init__(parent)
        
    def mouseDoubleClickEvent(self,ev):
        self.doubleClicked.emit()

# Widget to show/edit the source dictionary
class SourceDictWidget(QtWidgets.QTreeWidget):
    def __init__(self, parent = None):
        QtWidgets.QTreeWidget.__init__(self, parent)
        self.setColumnCount(3)
        self.setHeaderLabels(["Name", "Type", "Value"])
        self.setColumnWidth(0,130)
        self.setColumnWidth(1,90)
        # Check any edits which are made
        self.itemChanged.connect(self.checkEdits)
        # Value of an item prior to edit
        self.oldVal=''
        # Allowed variable data types
        self.varTypes=['str','int','float',
                       'array(str)',
                       'array(int)',
                       'array(float)']
        self.varConverts=[str,int,float,
                          lambda x: np.array(x.split(','),dtype=str),
                          lambda x: np.array(x.split(','),dtype=int),
                          lambda x: np.array(x.split(','),dtype=float)]
        
    # Keep default mouse presses
    def clickEvent(self,event):
        QtWidgets.QTreeWidget.mousePressEvent(self,event)
        # Clear selection if nothing clicked
        if self.itemAt(event.pos()) is None:
            self.clearSelection()

    def mousePressEvent(self, event):
        self.clickEvent(event)
        # If wanting to make add/remove an item (right click)
        if event.button() == QtCore.Qt.RightButton:
            self.createMenu(event)
    
    # Make a note of values prior to edits
    def mouseDoubleClickEvent(self,event):
        self.clickEvent(event)
        curItem=self.selectedItem()
        # Do nothing if no item selected
        if curItem is None:
            return
        # Allow group name to be edited, and the variable name/value
        curCol=self.currentColumn()
        if (curItem.parent() is None and curCol==0) or curCol!=1:
            self.oldVal=curItem.text(curCol)
            self.editItem(curItem,curCol)

    # Return list of the current group names
    def currentGroupNames(self):
        return[self.getName(groupItem) for groupItem in self.currentGroupItems()]
    
    # Return list of the current group items
    def currentGroupItems(self):
        return [self.topLevelItem(i) for i in range(self.topLevelItemCount())]
    
    # Return list of the variable names in a group item
    def currentVariableNames(self,groupItem):
        return [self.getName(varItem) for varItem in self.currentVariableItems(groupItem)]
    
    # Return list of variable items in a group item
    def currentVariableItems(self,groupItem):
        return [groupItem.child(i) for i in range(groupItem.childCount())]
    
    # Get the name of an item
    def getName(self,item):
        return str(item.text(0))

    # Generate widget for a new group 
    def addGroup(self,groupName='NewGroup'):
        # Check to see that the default name is not taken
        if groupName in self.currentGroupNames():
            print('Name "'+groupName+'" already taken, edit before adding')
            return
        groupItem=QtWidgets.QTreeWidgetItem([groupName,'',''])
        # Allow the item to be edited
        groupItem.setFlags(groupItem.flags() | QtCore.Qt.ItemIsEditable)
        self.addTopLevelItem(groupItem)
        return groupItem
    
    # Generate widget for a new variable within a group
    def addVariable(self,groupItem,varName='NewVariable',varVal=None):
        # Check to see that the default name is not taken
        if varName in self.currentVariableNames(groupItem):
            print('Name "'+varName+'" already taken, edit before adding')
            return
        varText,varType=self.getVarTextAndType(varVal)
        # Give default values for new variable
        varItem=QtWidgets.QTreeWidgetItem([varName,'',varText])
        # Allow the item to be edited
        varItem.setFlags(varItem.flags() | QtCore.Qt.ItemIsEditable)
        # Add the variable to the group
        groupItem.addChild(varItem)
        # Show what types are accepted
        comboBox=QtWidgets.QComboBox()
        comboBox.addItems(self.varTypes)
        # Set the current combo item
        comboBox.setCurrentIndex(self.varTypes.index(varType))
        self.setItemWidget(varItem,1,comboBox)
        # Connect to check edits if this value is changed
        comboBox.currentIndexChanged.connect(lambda: self.checkEdits(varItem,1))
        comboBox.highlighted.connect(lambda: self.setOldVal(comboBox.itemText(comboBox.currentIndex())))
    
    # Used to set the previous comboBox value
    def setOldVal(self,val):
        self.oldVal=val
    
    # Import and display a given source dictionary
    def showSourceDict(self,sourceDict):
        # Clear the old
        for groupItem in self.currentGroupItems():
           self.removeGroup(groupItem)
        # Load the new
        for groupName in sorted(sourceDict.keys()):
            groupItem=self.addGroup(groupName=str(groupName))
            for varName in sorted(sourceDict[groupName].keys()):
                self.addVariable(groupItem,varName=str(varName),
                                 varVal=sourceDict[groupName][varName])
    
    # Extract the displayed source dictionary
    def getSourceDict(self):
        sourceDict={}
        for groupItem in self.currentGroupItems():
            groupName=self.getName(groupItem)
            sourceDict[groupName]={self.getName(varItem):self.stringToVarType(varItem) 
                                   for varItem in self.currentVariableItems(groupItem)}
        return sourceDict
    
    # Remove currently selected group
    def removeGroup(self,item):
        self.takeTopLevelItem(self.indexFromItem(item).row())
    
    # Remove currently selected variable
    def removeVariable(self,item):
        item.parent().takeChild(self.indexFromItem(item).row())
        
    # Get the user visible value from the value and variable type 
    def getVarTextAndType(self,val):
        # If the value is none, convert to string
        if val is None:
            return '','str'
        # Check first if a numpy array
        if type(val)==type(np.array([])):
            for aType in ['float','int','str']:
                match='|S' if aType=='str' else aType
                if match in str(val.dtype):
                    if len(val.shape)==0:
                        return str(val),'array('+aType+')'
                    else:
                        return ','.join(val.astype(str)),'array('+aType+')'
        # Otherwise check float,int,str
        for i in range(3)[::-1]:
            try:
                self.varConverts[i](val)
                return str(val),self.varTypes[i]
            except:
                continue
        return str(val),'str'
        
    # Convert the variable value to its appropriate type
    def stringToVarType(self,item):
        comboBox=self.itemWidget(item,1)
        varType=comboBox.itemText(comboBox.currentIndex())
        convertFunc=self.varConverts[self.varTypes.index(varType)]
        if item.text(2).replace(' ','')=='':
            return None
        else:
            return convertFunc(item.text(2))
    
    # Revert to an older value
    def revertEdits(self,item,col,message):
        # Disconnect from changed edits, revert to old value, reconnect
        print(message)
        self.itemChanged.disconnect(self.checkEdits)
        item.setText(col,self.oldVal)
        self.itemChanged.connect(self.checkEdits)
    
    # Check recent edits for errors
    def checkEdits(self,item,col):
        # Ensure names are unique
        if col==0:
            # Make a list of the names currently present
            if item.parent() is None:
                takenNames=self.currentGroupNames()
            else:
                takenNames=self.currentVariableNames(item.parent())
            # If name already taken, revert to old
            if np.sum(np.array(takenNames,dtype=str)==str(item.text(col)))>1:
                self.revertEdits(item,col,'Name already taken')
        # Ensure type matches the value
        elif col==1 and item.parent() is not None:
            # Do not check if string (should always work)
            comboBox=self.itemWidget(item,1)
            if comboBox.itemText(comboBox.currentIndex())=='str':
                return
            try:
                self.stringToVarType(item)
            except Exception as error:
                # Disconnect from changed edits, revert to type string, reconnect
                print(str(error)+', reverting to old type')
                comboBox.setCurrentIndex(self.varTypes.index(self.oldVal))
        # Ensure value matches the type
        elif col==2 and item.parent() is not None:
            try:
                self.stringToVarType(item)
            except Exception as error:
                self.revertEdits(item,col,str(error)+', reverting to old value')
    
    # Get the currently selected item
    def selectedItem(self):
        items=self.selectedItems()
        if len(items)==0:
            return None
        else:
            return items[0]
    
    # Create pop up menu to add/remove groups/variables
    def createMenu(self,event, parent=None):
        self.menu=QtWidgets.QMenu(parent)
        self.menu.addAction('Add Group', self.addGroup)
        if self.selectedItem() is not None:
            if self.selectedItem().parent() is None:
                self.menu.addAction('Add Variable',lambda: self.addVariable(self.selectedItem()))
                self.menu.addSeparator()
                self.menu.addAction('Remove Group',lambda: self.removeGroup(self.selectedItem()))
            elif self.selectedItem().parent().parent() is None:
                self.menu.addAction('Add Variable',lambda: self.addVariable(self.selectedItem().parent()))
                self.menu.addSeparator()
                self.menu.addAction('Remove Variable',lambda: self.removeVariable(self.selectedItem()))
        self.menu.move(self.mapToGlobal(QtCore.QPoint(0,0))+event.pos())
        self.menu.show() 



from __future__ import print_function

import numpy as np        
import time

# Go to the next pick type (in alphabetical order)
def togglePickMode(*args,**kwargs):
    curMode=args[0]
    availPickModes=sorted([str(key) for key in args[1].keys()])
    if curMode in availPickModes:
        idx=availPickModes.index(curMode)
    else:
        idx=0
    idx=(idx+1)%len(availPickModes)
    return availPickModes[idx]
    
# Go to the first or last page
def goToPage(*args,**kwargs):
    if kwargs['goToPage']=='last':
        pageNumber=99999
    else:
        pageNumber=0
    return pageNumber
        
# Toggle trace coloring between channels
def toggleTraceColor(curAssign):
    if 'lowlight' not in curAssign.keys():
        return {'lowlight':['*Z'],'highlight':['*1','*2','*E','*N']}
    if '*Z' in curAssign['lowlight']:
        return {'highlight':['*Z'],'lowlight':['*1','*2','*E','*N']}
    else:
        return {'lowlight':['*Z'],'highlight':['*1','*2','*E','*N']}
        
# Alternate the three components colors
def alternateTraceColor(curAssign):
    # If nothing assigned, gib
    if 'alt1' not in [key for key in curAssign.keys()]:
        return {'alt1':['*Z'],'alt2':['*2','*N'],'alt3':['*1','*E']}
    elif '*Z' in curAssign['alt1']:
        return {'alt2':['*Z'],'alt3':['*2','*N'],'alt1':['*1','*E']}
    elif '*Z' in curAssign['alt2']:
        return {'alt3':['*Z'],'alt1':['*2','*N'],'alt2':['*1','*E']}
    elif '*Z' in curAssign['alt3']:
        return {'alt1':['*Z'],'alt2':['*2','*N'],'alt3':['*1','*E']}

# Alternate the colors of the station color assignment
def alternateStaColorAssign(curAssign,staSort):
    # Do nothing if no stations to color
    if len(staSort)==0:
        return '$pass'
    # If given the staMeta instead of the staSort
    if len(staSort.shape)==2:
        staSort=staSort[:,0]
    # If nothing assigned, make one
    num=0
    if curAssign=={}:
        pass
    elif staSort[0] in curAssign['alt1']:
        num=1
    curAssign={'alt1':[],'alt2':[]}
    # Add all the new stations to the coloring assignment
    for i in np.arange(len(staSort)):
        if i%2==num:
            curAssign['alt1'].append(staSort[i])
        else:
            curAssign['alt2'].append(staSort[i])          
    return curAssign
    
# Make new pick dir
def newPickDir(mainPath):
    return mainPath+'/NewPickDir!'

# Go this pick file
def goToPickFile():
    return '2_20150907.051000.000000.picks'
    
# Toggle between sources
def toggleTestSources(curTag):
    if curTag=='testing':
        return 'testing2'
    else:
        return 'testing'
        
# Move forward in time by a portion of current range
def panPercent(*args,**kwargs):
    timeRange=args[0]
    delta=timeRange[1]-timeRange[0]
    if kwargs['direct']=='backward':
        delta*=-1.0
    timeRange[0]+=kwargs['percent']/100.0*delta
    timeRange[1]+=kwargs['percent']/100.0*delta
    return timeRange
    
# Zoom in/out by a certain percent, centered over current position
def zoomTraceY(yTraceRanges,percent=100):
    avg=np.mean(yTraceRanges,axis=1)
    halfWidth=(np.diff(yTraceRanges,axis=1)/2.0)[:,0]
    yTraceRanges[:,0]=avg-halfWidth*percent/100.0
    yTraceRanges[:,1]=avg+halfWidth*percent/100.0
    return yTraceRanges
        
# Return some random set of current and previous events
def randomEves(staLoc):
    # Do nothing if no stations have been added
    if len(staLoc)==0:
        return '$pass','$pass'
    staLoc=staLoc[:,1:].astype(float)
    # Make a random distribution of events bounded by the station limits
    xmin,xmax=np.min(staLoc[:,0]),np.max(staLoc[:,0])
    ymin,ymax=np.min(staLoc[:,1]),np.max(staLoc[:,1])
    xArr=np.random.rand(100)*(xmax-xmin)+xmin
    yArr=np.random.rand(100)*(ymax-ymin)+ymin
    zArr=np.ones(100)
    prevArr=np.vstack((zArr,xArr,yArr,zArr,zArr)).T
    curArr=prevArr[:3,:]
    return curArr,prevArr     

# Test for the image hot variable
def randImage(timeRange):
    tDelta=0.2
    xLen=int((timeRange[1]-timeRange[0])/tDelta)
    data=np.random.rand(xLen,10)
    if 0 in data.shape:
        return '$pass'
    return {'data':data,'t0':timeRange[0],'tDelta':tDelta}

# To print out a specific variable (Testing)
def printMe(arg):
    print(arg)
    
# To test passing of the custom dictionary variable
def testCustDict(custDict):
    if 'test123' not in custDict.keys():
        custDict['test123']=0
    else:
        custDict['test123']=(custDict['test123']+1)%5
    print(custDict)
    return custDict

# Threading test   
def waitAction(waitTime=1):
    time.sleep(waitTime)
    
# Matplotlib external plot test
def testMatplotlib():
    import matplotlib.pyplot as plt
    plt.ion()
    plt.plot([1,2],[1,2])
    plt.show()
    
# Edit the source dictionary
def editSourceDict(sourceDict):
    sourceDict['aGroup']={'newVal':np.array([0,0,99,99.999]),
                          'aaa':'haha'}
    return sourceDict

# Test polygon plotting
def editPolygon(polygon):
    polygon=np.vstack((polygon,polygon[-2:]+1))
    return polygon


from obspy.core.inventory import (Inventory, Network, Station, Channel, Site, 
                                  Response, InstrumentSensitivity, Frequency,
                                  PolesZerosResponseStage,CoefficientsTypeResponseStage)
from obspy.core.util.obspy_types import FloatWithUncertaintiesAndUnit
from obspy.io.xseed import Parser
from obspy import UTCDateTime
from obspy.clients.fdsn.client import Client
import numpy as np

# This file contains a group of functions which can be used/edited to
# station metadata into the station XML format

# Example to extract a station XML file from an FDSN client
def getXmlFromIRIS(network=None):
    if network is None:
        return
    client=Client("IRIS")
    # Use level=response to get channel information...
    # ...which holds the location code necessary for station referencing
    inv=client.get_stations(network=network,level='response')
    inv.write('./'+network+'.xml',format='stationxml',validate=True)
    
getXmlFromIRIS(network='NX')
    
# Convert a list of two arrays into one which has uncertainty attached
def genArrWithUncertainty(vals,errs):
    outArr=[FloatWithUncertaintiesAndUnit(val,lower_uncertainty=err,upper_uncertainty=err) 
            for val,err in zip(vals,errs)]
    return outArr

# Ensure that the array is 1D (could be 0D)
def setArrDim(arr):
    if len(arr.shape)==0:
        arr=np.array([arr],dtype=type(arr))
    return arr

# Collect a station object from a dataless seed station block
def getStation(stationBlock,units,transFuncs):
    ## Should probably do a check up here to see that the order given in block is consistent ##
    for entry in stationBlock:
        if entry.name=='Station Identifier':
#            print 'NewStation!',entry.station_call_letters
            staDict={'code':entry.station_call_letters,
                    'latitude':entry.latitude,
                    'longitude':entry.longitude,
                    'elevation':entry.elevation,
                    'channels':[],
                    'site':Site(entry.site_name),
                    'creation_date':UTCDateTime(entry.start_effective_date), # Allows for save
                    'start_date':UTCDateTime(entry.start_effective_date),
                    'end_date':UTCDateTime(entry.end_effective_date)}
            staNetCode=entry.network_code
        # If found a new channel, reset the stages
        elif entry.name=='Channel Identifier':
#            print 'NewChannel!',entry.channel_identifier
            stages=[]
            chaDict={'code':entry.channel_identifier,
                     'location_code':entry.location_identifier,
                     'latitude':entry.latitude,
                     'longitude':entry.longitude,
                     'elevation':entry.elevation,
                     'depth':entry.local_depth,
                     'sample_rate':entry.sample_rate,
                     'start_date':UTCDateTime(entry.start_date),
                     'end_date':UTCDateTime(entry.end_date),
                     'azimuth':entry.azimuth,
                     'dip':entry.dip}
        #code, location_code, latitude, longitude, elevation, depth
        # If on a new stage, set up the dictionary again
        # ...paz stage
        elif entry.name=='Response Poles and Zeros':
            # Get units
            stageReqs={}
#            print entry.name,entry.stage_sequence_number
#            print entry
#            quit()
            stageReqs['input_units']=units[entry.stage_signal_input_units]
            stageReqs['output_units']=units[entry.stage_signal_output_units]
            # Collect the poles and zeros
            lastType='paz'
            if entry.number_of_complex_zeros==0:
                zeros=np.array([],dtype=float)
            else:
                zeros=np.array(entry.real_zero,dtype=float)+np.array(entry.imaginary_zero,dtype=float)*1j
            if entry.number_of_complex_poles==0:
                poles=np.array([],dtype=float)
            else:
                poles=np.array(entry.real_pole,dtype=float)+np.array(entry.imaginary_pole,dtype=float)*1j
            # Form the paz response dictionary (also ensure arrays are 1D)
            pazDict={'pz_transfer_function_type':transFuncs[entry.transfer_function_types],
                     'normalization_factor':entry.A0_normalization_factor,
                     'normalization_frequency':entry.normalization_frequency,
                     'zeros':setArrDim(zeros),
                     'poles':setArrDim(poles)}
        # ...coeff stage
        elif entry.name=='Response Coefficients':
            # Get units
            stageReqs={}
#            print entry.name,entry.stage_sequence_number
            stageReqs['input_units']=units[entry.signal_input_units]
            stageReqs['output_units']=units[entry.signal_output_units]
            # Collect the coefficients
            lastType='coef'
            if entry.number_of_denominators==0:
                denom=np.array([],dtype=float)
                denomErr=np.array([],dtype=float)
            else:
                denom=np.array(entry.denominator_coefficient,dtype=float)
                denomErr=np.array(entry.denominator_error,dtype=float)
            if entry.number_of_numerators==0:
                numer=np.array([],dtype=float)
                numerErr=np.array([],dtype=float)
            else:
                numer=np.array(entry.numerator_coefficient,dtype=float)
                numerErr=np.array(entry.numerator_error,dtype=float)
            # Convert these arrays into lists of numbers which have uncertainty (also ensure arrays are 1D)
            denomArr=genArrWithUncertainty(setArrDim(denom),setArrDim(denomErr))
            numerArr=genArrWithUncertainty(setArrDim(numer),setArrDim(numerErr))
            # Form the coeefficient response dictionary
            coefDict={'cf_transfer_function_type':transFuncs[entry.response_type],
                      'numerator':numerArr,
                      'denominator':denomArr}
        # Get the decimation sampling info
        elif entry.name=='Decimation':
#            print entry.name,entry.stage_sequence_number
            stageReqs['decimation_input_sample_rate']=Frequency(entry.input_sample_rate)
            stageReqs['decimation_factor']=entry.decimation_factor
            stageReqs['decimation_offset']=entry.decimation_offset
            stageReqs['decimation_delay']=FloatWithUncertaintiesAndUnit(entry.estimated_delay)
            stageReqs['decimation_correction']=FloatWithUncertaintiesAndUnit(entry.correction_applied)
        # Get the stage sensitivity
        elif entry.name=='Channel Sensitivity Gain':
#            print entry.name,entry.stage_sequence_number
            if entry.stage_sequence_number!=0:
                stageReqs['stage_sequence_number']=entry.stage_sequence_number
                stageReqs['stage_gain']=entry.sensitivity_gain
                stageReqs['stage_gain_frequency']=entry.frequency
                # See what type of stage this was
                if lastType=='paz':
                    pazDict.update(stageReqs)
                    stages.append(PolesZerosResponseStage(**pazDict))
                else:
                    coefDict.update(stageReqs)
                    stages.append(CoefficientsTypeResponseStage(**coefDict))
            # If on the last stage, send off the collected stage info
            else:
                if len(stages)>0:
                    instrSens=InstrumentSensitivity(entry.sensitivity_gain,entry.frequency,
                                                    stages[0].input_units,stages[-1].output_units)
                    # Finalize the channel dictionary, and append this channel to the station dictionary
                    chaResp=Response(response_stages=stages,
                                     instrument_sensitivity=instrSens)
                    chaDict['response']=chaResp
                staDict['channels'].append(Channel(**chaDict))
    # Return the stations to the list of stations (also track the network code)
    return Station(**staDict),staNetCode
    
# Convert a dataless seed to a station XML file
def dataless2stationXml(datalessFileName,xmlFileName):
    # Read the dataless seed file
    sp = Parser(datalessFileName)    
    
    # Collect all potential unit abbreviations
    units={}
    #genAbbrev={}
    for entry in sp.abbreviations:
        if entry.name=='Units Abbreviations':
            units[entry.unit_lookup_code]=entry.unit_name
    #    elif entry.name=='Generic Abbreviation':
    #        genAbbrev[entry.abbreviation_lookup_code]=entry.abbreviation_description

    # Make a look-up dictionary for the transfer functions
    transFuncs={'A':'LAPLACE (RADIANS/SECOND)',
                'B':'ANALOG (HERTZ)',
                'C':'COMPOSITE',
                'D':'DIGITAL (Z-TRANSFORM)'}
    
    # Collect each of the stations objects
    stations=[]
    staNetCodes=[]    
    for stationBlock in sp.stations:
        station,staNetCode=getStation(stationBlock,units,transFuncs)
        stations.append(station)
        staNetCodes.append(staNetCode)
        
    # For each of the unique networks codes, collect the stations which relate to it
    networks=[]
    staNetCodes=np.array(staNetCodes)
    unqNets=np.unique(staNetCodes)
    for aNet in unqNets:
        netStas=[stations[arg] for arg in np.where(staNetCodes==aNet)[0]]
        networks.append(Network(aNet,stations=netStas))
    
    # Finally turn this into an inventory and save
    inv=Inventory(networks,'Lazylyst')
    inv.write(xmlFileName,format='stationxml',validate=True)
    

#dataless2stationXml('NX.dataless','NX_Full2.xml')






# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'BasePen.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_basePenDialog(object):
    def setupUi(self, basePenDialog):
        basePenDialog.setObjectName("basePenDialog")
        basePenDialog.resize(393, 512)
        self.verticalLayout = QtWidgets.QVBoxLayout(basePenDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tpTable = QtWidgets.QTableWidget(basePenDialog)
        self.tpTable.setObjectName("tpTable")
        self.tpTable.setColumnCount(0)
        self.tpTable.setRowCount(0)
        self.verticalLayout.addWidget(self.tpTable)

        self.retranslateUi(basePenDialog)
        QtCore.QMetaObject.connectSlotsByName(basePenDialog)

    def retranslateUi(self, basePenDialog):
        _translate = QtCore.QCoreApplication.translate
        basePenDialog.setWindowTitle(_translate("basePenDialog", "basePen"))




from __future__ import print_function

import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from obspy.taup import TauPyModel
model=TauPyModel(model="iasp91")
import pickle

# 3rd order polynomial fixed at 0,0
def poly3_Fixed(x,a,b,c):
    return np.clip(a*x**1+b*x**2+c*x**3,0,2000)

# 3rd order polynomial
def poly3(x,a,b,c,d):
    return np.clip(a*x**1+b*x**2+c*x**3+d,0,2000)

# Generate the epicentral location tests points
def getEpiPoints():
    degSpc=0.01
    testDegs=np.arange(0,180+0.5*degSpc,degSpc,dtype=float)
    pTimes,sTimes=[],[]
    for deg in testDegs:
        if deg%1==0:
            print(deg)
        # Get the earliest p and s arrival time
        arrivals=model.get_travel_times(source_depth_in_km=5,distance_in_degree=deg,
                                        phase_list=['ttp','tts'])
        pTimes.append(next((entry.time for entry in arrivals if entry.purist_name[0] in ['P','p']),None))
        sTimes.append(next((entry.time for entry in arrivals if entry.purist_name[0] in ['S','s']),None))
    outArr=np.array([testDegs,np.array(pTimes),np.array(sTimes)]).T
    np.save('globalTT',outArr)

# Generate the hypocentral location test points
def getDepthPoints():
    degSpc=0.01
    testDegs=np.arange(0,45+0.5*degSpc,degSpc,dtype=float)
    testDeps=np.arange(0,200.1,5)
    fullArr=[]
    for deg in testDegs:
        print(deg)
        pTimes,sTimes=[],[]
        for dep in testDeps:
            # Get the earliest p and s arrival time
            arrivals=model.get_travel_times(source_depth_in_km=dep,distance_in_degree=deg,
                                            phase_list=['ttp','tts'])
            pTimes.append(next((entry.time for entry in arrivals if entry.purist_name[0] in ['P','p']),None))
            sTimes.append(next((entry.time for entry in arrivals if entry.purist_name[0] in ['S','s']),None))
        fullArr.append(np.array([testDeps,np.array(pTimes),np.array(sTimes)]).T)
    np.save('globalTT_vDep_1',np.array(fullArr,dtype=float))

# Calculate the parameters in TT(EpiDist) to be used at specific degrees
def getEpiParams():
    outArr=np.load('globalTT.npy')
    globP,pcov=curve_fit(poly3_Fixed, outArr[:,0], outArr[:,1])
    globS,pcov=curve_fit(poly3_Fixed, outArr[:,0], outArr[:,2])
    
    # For a suite of small limits, calculate the parameters for refined searching
    buff=2.5
    spacing=1.0
    startDep=5 # Source depth used when calculating epi points
    plt.plot(outArr[:,0],outArr[:,1],'c--',lw=2)
    plt.plot(outArr[:,0],outArr[:,2],'b--',lw=2)
    outDict={'globP':globP, # Parameters for fitting the global TT values of P-wave
             'globS':globS, # Parameters for fitting the global TT values of S-wave
             'buff':buff,   # Buffer about each degree used to fit for the refined parameters 
             'spacing':spacing, # Spacing between degrees for refined parameter calculation
             'startDep':startDep}
    pParams,sParams=[],[]
    for deg in np.arange(0,180.0+0.1*spacing,spacing):
        print(deg)
        aMin=np.max([0,deg-buff])
        aMax=np.min([180,deg+buff])
        wantArgs=np.where((outArr[:,0]>=aMin)&(outArr[:,0]<=aMax))[0]
        paramP,pcov=curve_fit(poly3_Fixed, outArr[wantArgs,0], outArr[wantArgs,1],globP)
        paramS,pcov=curve_fit(poly3_Fixed, outArr[wantArgs,0], outArr[wantArgs,2],globS)
        plt.plot(outArr[wantArgs,0],poly3_Fixed(outArr[wantArgs,0],*paramP),'g',lw=0.5)
        plt.plot(outArr[wantArgs,0],poly3_Fixed(outArr[wantArgs,0],*paramS),'r',lw=0.5)
        pParams.append(paramP)
        sParams.append(paramS)
    
    outDict['pParams']=np.array(pParams)
    outDict['sParams']=np.array(sParams)
    pickle.dump( outDict, open('epiTTparams.pickle','wb'))
    
    plt.plot(outArr[:,0],poly3_Fixed(outArr[:,0],*globP),'k')
    plt.plot(outArr[:,0],poly3_Fixed(outArr[:,0],*globS),'k')
    plt.show()
    
# Calculate the parameters in TT(Depth) to be used at specific degrees
def getDepParams():
    data1=np.load('globalTT_vDep_1.npy')
    data2=np.load('globalTT_vDep_2.npy')
    data3=np.load('globalTT_vDep_3.npy')
    data4=np.load('globalTT_vDep_4.npy')
    
    data2=data2[1:,:,:]
    data3=data3[1:,:,:]
    data4=data4[1:,:,:]
    
    data=np.vstack((data1,data2,data3,data4))
    # Assumes starts at 0 degrees and ends at 180
    outDict={'spacing':0.01}
    
    pParams,sParams=[],[]
    for entry in data:
        paramP,pcov=curve_fit(poly3, entry[:,0], entry[:,1])
        paramS,pcov=curve_fit(poly3, entry[:,0], entry[:,2])
        pParams.append(paramP)
        sParams.append(paramS)
    outDict['pParams']=np.array(pParams)
    outDict['sParams']=np.array(sParams)
    pickle.dump( outDict, open('depTTparams.pickle','wb'))
    
#getDepParams()


from __future__ import print_function

import numpy as np

from Plugins.Locate import getVelDelay,getPickData,simpleLocatorFunc
from Plugins.GlobalLocate import sph2xyz,vecsAngle

# Return the stations in alphabetical order
def staSortAlph(staSort):
    return np.sort(staSort)
    
# Return the station based on the current pick ordering
def staSortPickTime(staSort,pickSet,pickMode):
    # Tertiary sorting is alphabetical
    staSort=np.sort(staSort)
    # If no picks, return alphabetical
    if len(pickSet)==0:
        return staSort
    # Secondary sorting is all other pick types (add a large delay)
    maxT=np.max(pickSet[:,2].astype(float))
    offset=maxT-np.min(pickSet[:,2].astype(float))+1
    args=np.where(pickSet[:,1]!=pickMode)
    pickSet[args,2]=(pickSet[args,2].astype(float)+offset).astype(str)
    pos=[]
    for i,sta in enumerate(staSort):
        if sta in pickSet[:,0]:
            pos.append(np.min(pickSet[np.where(pickSet[:,0]==sta),2].astype(float)))
        else:
            pos.append(maxT+offset+i+1)
    return staSort[np.argsort(pos)]

# Return the station based on the distance from the current event
def staSortDist(staSort,staLoc,mapCurEve,staProjStyle):
    # If no event has been located, do not change anything
    if len(mapCurEve)==0:
        return '$pass'
    # Secondary sorting is alphabetical
    staSort=np.sort(staSort)
    # Replace Lons,Lats,Elev with X,Y,Z on sphere if no projection...
    # ...also split array into the string and float components
    staNames=list(staLoc[:,0])
    if staProjStyle=='None':
        staLoc=sph2xyz(staLoc[:,1:3].astype(float))
        eveLoc=sph2xyz(mapCurEve[:,1:3].astype(float))[0]
    else:
        staLoc=staLoc[:,1:].astype(float)
        eveLoc=mapCurEve[0,1:4]
    # Loop through each station, getting its distance from the current event...  
    pos=[]
    for i,sta in enumerate(staSort):
        if sta in staNames:
            staXyz=staLoc[staNames.index(sta),:]   
            # ...if projected, then calculate euclidean distance
            if staProjStyle!='None':
                dist=np.sum((staXyz-eveLoc)**2)**0.5
            # ...otherwise calculate the angle between stations and the event (depth ignored)
            else:
                dist=vecsAngle(staXyz,eveLoc)
            pos.append(dist)
        else:
            pos.append(-1*(i+1))
    # Reassign the "pos" of stations which did not have a station metadata present
    pos=np.array(pos,dtype=float)
    args=np.where(pos<0)
    pos[args]=pos[args]*-1+np.max(pos)
    return staSort[np.argsort(pos)]
   
# Return the station based on the residual (largest first)...
# ...currently works only with SimpleLocator
def staSortResidual(staSort,staLoc,sourceDict,pickSet,mapCurEve,staProjStyle):
    # Secondary sorting is alphabetical
    staSort=np.sort(staSort)
    # Get the velocity and delay info
    vdInfo=getVelDelay(sourceDict)
    # If there was no vpInfo defined, or no event has been located, return alphabetical
    # Also return if not using a projection (simpleLocator requires it)
    if vdInfo=='$pass' or len(mapCurEve)==0 or staProjStyle=='None':
        return staSort
    data,stas=getPickData(pickSet,staLoc,vdInfo)
    # If there was none of the wanted pick types, return alphabetical
    if len(data)==0:
        return staSort
    x0,y0,z0,t0=mapCurEve[0,1:]
    ti=simpleLocatorFunc(data[:,:5],x0,y0,z0,t0)
    # Get the difference with the actual arrival times
    resids=np.abs(ti-data[:,5])
    offset=np.max(resids)+1
    # Loop through each station, getting its max residual value from all phases   
    pos=[]
    for i,sta in enumerate(staSort):
        if sta in stas:
            pos.append(-1*np.max(resids[np.where(stas==sta)]))
        else:
            pos.append(i+offset)
    return staSort[np.argsort(pos)]
    
# Reverse the current station sorting
def staSortReverse(staSort):
    return staSort[::-1]


# Author: Andrew.M.G.Reynen
from __future__ import print_function, division
import time

import numpy as np
from PyQt5 import QtGui,QtCore,QtWidgets
import pyqtgraph as pg

# Scatter Item class, but allow double click signal
class CustScatter(pg.ScatterPlotItem):
    doubleClicked=QtCore.Signal(object)
    
    def __init__(self, *args, **kwargs):
        super(CustScatter, self).__init__(*args, **kwargs)
    
    # Set the clicked points upon double clicking
    def mouseDoubleClickEvent(self,ev):
        if ev.button() == QtCore.Qt.LeftButton:
            # Set the clicked position
            self.clickPos=np.array([ev.pos().x(),ev.pos().y()])
            # See which points were near the click position
            self.ptsClicked=self.pointsAt(ev.pos())
            # Emit the scatter plot
            self.doubleClicked.emit(self)
            ev.accept()
            
# Verticies on the polygon
class RoiHandle(pg.graphicsItems.ROI.Handle):
    handleMenuSignal=QtCore.Signal(float)
    def __init__(self, *args, **kwargs):
        super(RoiHandle, self).__init__(*args, **kwargs)

    def mouseClickEvent(self, ev):
        # Right-click cancels drag
        if ev.button() == QtCore.Qt.RightButton and self.isMoving:
            self.isMoving = False  # Prevent any further motion
            self.movePoint(self.startPos, finish=True)
            ev.accept()
        elif int(ev.button() & self.acceptedMouseButtons()) > 0:
            ev.accept()
            if ev.button() == QtCore.Qt.RightButton and self.deletable:
                self.raiseContextMenu(ev)
            self.sigClicked.emit(self, ev)
        else:
            ev.ignore()        
    
    # Make some edits to default pg Handle
    def raiseContextMenu(self, ev):
        menu = self.scene().addParentContextMenus(self, self.getMenu(), ev)
        # Remove the unwanted actions
        for action in menu.actions():
            if str(action.text())!='Remove handle':
                menu.removeAction(action)
        # Make sure it is still ok to remove this handle
        removeAllowed = all([r.checkRemoveHandle(self) for r in self.rois])
        self.removeAction.setEnabled(removeAllowed)
        pos = ev.screenPos()
        # Note when the menu was added
        self.handleMenuSignal.emit(time.time())
        menu.popup(QtCore.QPoint(pos.x(), pos.y()))  

# Editable polygon
class RoiPolyLine(pg.PolyLineROI):
    handleMenuSignal=QtCore.Signal(float)
    def __init__(self, *args, **kwargs):
        super(RoiPolyLine, self).__init__(*args, **kwargs)
    
    # Make use of the edited pg Handle "RoiHandle"  
    def addHandle(self, info, index=None):
        # If a Handle was not supplied, create it now
        if 'item' not in info or info['item'] is None:
            h = RoiHandle(self.handleSize, typ=info['type'], pen=self.handlePen, parent=self)
            h.setPos(info['pos'] * self.state['size'])
            info['item'] = h
        else:
            h = info['item']
            if info['pos'] is None:
                info['pos'] = h.pos()
        # Connect the handle to this ROI
        h.connectROI(self)
        if index is None:
            self.handles.append(info)
        else:
            self.handles.insert(index, info)
        h.setZValue(self.zValue()+1)
        h.sigRemoveRequested.connect(self.removeHandle)
        h.handleMenuSignal.connect(self.relayMenuTime)
        self.stateChanged(finish=True)
        return h
    
    # Relay the time of a menu pop up on a handle 
    def relayMenuTime(self,aTime):
        self.handleMenuSignal.emit(aTime)

# Widget for seeing what data times are available, and sub-selecting pick files
class MapWidget(pg.GraphicsLayoutWidget):
    doubleClicked=QtCore.Signal()
    updatePolygonPenSignal=QtCore.Signal()
    
    def __init__(self, parent=None):
        super(MapWidget, self).__init__(parent)
        # Add the map to the view, and turn off some of the automatic interactive properties
        self.map=self.addPlot()
        self.map.setMenuEnabled(enableMenu=False)
        self.map.hideButtons()
        self.stas=[] # The station names for the station spot items
        self.staItem=None # The station scatter item
        self.selectSta=None # Which station is currently selected
        self.curEveItem=None # The current event scatter item
        self.prevEveItem=None # The previous event scatter item 
        self.clickPos=[0,0] # Last double clicked position
        self.clickDownPos=False # Mouse position upon clicking down
        # Add in the hovered over station label
        self.hoverStaItem=pg.TextItem(text='',anchor=(0.5,1))
        self.hoverStaItem.hide()
        self.map.addItem(self.hoverStaItem)
        # Show station text when hovering over the station symbol
        self.scene().sigMouseMoved.connect(self.onHover)
        # Holder for editable polygon
        self.polygon=None
        self.handleMenuTime=0

    # Track where the mouse was pressed down
    def mousePressEvent(self, event):
        pg.GraphicsLayoutWidget.mousePressEvent(self,event)
        self.clickDownPos=event.pos()
    
    # If no dragging occcured and handle menu was not created, create menu to add polygon
    def mouseReleaseEvent(self,event):
        pg.GraphicsLayoutWidget.mouseReleaseEvent(self,event)
        if (event.pos()==self.clickDownPos and event.button() == QtCore.Qt.RightButton and
            time.time()-self.handleMenuTime>0.05):
            self.createMenu(event)
            
    # Add some functionality to default hover events...
    # ...see which station is being hovered over
    def onHover(self,pixPoint):
        if len(self.stas)==0:
            return
        # Get the nearby points
        mousePoint=self.staItem.mapFromScene(pixPoint)
        nearPoints = self.staItem.pointsAt(mousePoint)
        # Grab the nearest ones station code and update the 
        if len(nearPoints)==0:
            self.hoverStaItem.hide()
        else:
            mousePos=np.array([mousePoint.x(),mousePoint.y()])
            sta=self.getSelectSta(mousePos,nearPoints)
            self.hoverStaItem.setText(sta)
            self.hoverStaItem.setPos(mousePoint)
            self.hoverStaItem.show()
        
    # Handle double click events to the station scatter item
    def dblClicked(self,staScat):
        self.clickPos=list(staScat.clickPos)
        # Update the nearby station which was clicked
        self.selectSta=self.getSelectSta(staScat.clickPos,staScat.ptsClicked)
        # Forward the double clicked signal to main window
        self.doubleClicked.emit()
        
    # Create pop up menu to add/remove groups/variables
    def createMenu(self,event, parent=None):
        pixelPos=event.pos()
        self.menu=QtWidgets.QMenu(parent)
        if self.polygon is None:
            self.menu.addAction('Add polygon', lambda:self.addDefaultPolygon(pixelPos))
        else:
            self.menu.addAction('Remove polygon',self.removePolygon)
        self.menu.move(self.mapToGlobal(QtCore.QPoint(0,0))+pixelPos)
        self.menu.show()
    
    # Load the polygon onto the map
    def loadPolygon(self,vertices):
        self.polygon=RoiPolyLine(vertices, closed=True)
        self.polygon.handleMenuSignal.connect(self.setHandleMenuTime)
        self.map.addItem(self.polygon)
        
    # If polygon created via GUI, make a box near clicked position
    def addDefaultPolygon(self,pixelPos):
        viewBox=self.map.getViewBox()
        # Convert from pixel position
        clickPos=viewBox.mapSceneToView(pixelPos)
        clickPos=np.array([clickPos.x(),clickPos.y()])
        # Get the map axis lengths
        xLen,yLen=np.diff(np.array(viewBox.viewRange(),dtype=float),axis=1)*0.2
        xLen,yLen=xLen[0],yLen[0]
        vertices=[clickPos+np.array(extra) for extra in [[0,0],[0,yLen],[xLen,yLen],[xLen,0]]]
        self.loadPolygon(vertices)
        # Send off signal for the polygons pen to be updated
        self.updatePolygonPenSignal.emit()
        
    # Remove polygon from the map
    def removePolygon(self):
        if self.polygon is not None:
            self.map.removeItem(self.polygon)
        self.polygon=None
    
    # Get the positions of the handles on the polygon
    def getPolygonVertices(self):
        if self.polygon is None:
            return np.empty((0,2),dtype=float)
        positions=[handle['pos'] for handle in self.polygon.handles]
        vertices=np.array([[entry.x(),entry.y()] for entry in positions])
        return vertices
    
    # Set the time the handle menu was created
    def setHandleMenuTime(self,aTime):
        self.handleMenuTime=aTime
        
    # Function to return the nearest station to the current moused point
    def getSelectSta(self,mousePos,nearPoints):
        if len(nearPoints)>0:
            selPos=np.array([[point.pos().x(),point.pos().y()] for point in nearPoints])
            selPoint=nearPoints[np.argmin(np.sum((selPos-mousePos)**2))]
            return str(self.stas[np.where(self.staItem.points()==selPoint)[0][0]])
        else:
            return None
        
    # Load the new station meta data
    def loadStaLoc(self,staLoc,colorAssign,
                   staSize,staDep,init):
        # Enable the autoscaling temporarily
        self.map.vb.enableAutoRange(enable=True)
        # Hide the hovered station label
        self.hoverStaItem.hide()
        # Clear away the old station spots
        if self.staItem!=None:
            self.map.removeItem(self.staItem)
        # Reset the double clicked station, if reloading the station file entirely
        if init:
            self.selectSta=None
        # Remove the old polygon
        self.removePolygon()
        # Generate the station items
        if len(staLoc)==0:
            self.stas=[]
        else:
            self.stas=staLoc[:,0]
        # Get the brush values to be assigned
        brushArr=[]
        for sta in self.stas:
            color=colorAssign[sta]
            brushArr.append(pg.mkBrush(color.red(),color.green(),color.blue(),200))
        staScatter=CustScatter(size=staSize*2,symbol='t1',pen=pg.mkPen(None))
        staScatter.setZValue(staDep)
        # Add in the points
        if len(staLoc)!=0:
            staScatter.addPoints(x=staLoc[:,1], y=staLoc[:,2], brush=brushArr)
        # Give some clicking ability to the stations
        staScatter.doubleClicked.connect(self.dblClicked) # For any point being clicked
        # Add the station scatter items
        self.map.addItem(staScatter)
        self.staItem=staScatter
        # Disable autoscaling the for map items
        self.map.vb.enableAutoRange(enable=False)
        
    # Load a set of event points
    def loadEvePoints(self,eveMeta,eveType):
        if eveType=='cur':
            item,alpha=self.curEveItem,200
        else:
            item,alpha=self.prevEveItem,160
        # Remove the previous item
        if item is not None:
            self.map.removeItem(item)
        # Add in all of the new points, size & color set in different function
        scatter=pg.ScatterPlotItem(size=1,symbol='o',pen=pg.mkPen(None),brush=pg.mkBrush(0,0,0,alpha))
        scatter.addPoints(x=eveMeta[:,1],y=eveMeta[:,2])
        self.map.addItem(scatter)
        # Update the scatter item reference
        if eveType=='cur':
            self.curEveItem=scatter
        else:
            self.prevEveItem=scatter
        
    # Update the event pen values
    def updateEvePen(self,evePen,eveType):
        col,size,dep=evePen
        col=QtGui.QColor(QtGui.QColor(col))
        if eveType=='cur':
            item=self.curEveItem
        else:
            item=self.prevEveItem
        if item!=None:
            item.setBrush(pg.mkBrush(col.red(),col.green(),col.blue(),200))
            item.setSize(size*2)
            item.setZValue(dep)
    
    # Update the pen on the polygon
    def updatePolygonPen(self,evePen):
        if self.polygon is None:
            return
        col,width,dep=evePen
        col=QtGui.QColor(QtGui.QColor(col))
        self.polygon.setPen(col,width=width)
        self.polygon.setZValue(dep)
        self.polygon.setVisible(width!=0)
        # Set the depth of the handles to be slightly higher
        for handle in self.polygon.handles:
            handle['item'].setZValue(dep+2)
        
    
    # Change the pen of the axis and axis labels
    def setPen(self,pen):
        for item in self.items():
            if type(item)==pg.graphicsItems.AxisItem.AxisItem:
                item.setPen(pen)


from __future__ import print_function
import pickle

import numpy as np
from scipy.optimize import curve_fit

# Reference each pick to a station index
def getPickStaIdxs(pickSet,staNames):
    pickStas,pickIdx=np.unique(pickSet[:,0],return_inverse=True)
    pickStaIdxs=np.ones(len(pickIdx),dtype=int)*-1
    for pos in np.unique(pickIdx):
        sta=pickStas[pos]
        if sta in staNames:
            posArgs=np.where(pickIdx==pos)[0]
            pickStaIdxs[posArgs]=np.where(staNames==sta)[0][0]
    # Remove any picks which did not have station metadata
    keepArgs=np.where(pickStaIdxs!=-1)[0]
    pickStaIdxs=pickStaIdxs[keepArgs]
    pickSet=pickSet[keepArgs]
    return pickSet,pickStaIdxs

# Convert spherical coord numpy array to cartesian
# [[Lon1,Lat1],[Lon2,Lat2],...]
def sph2xyz(p):
    p[:,1]+=90
    # Convert from deg to rad
    p=np.pi*p/180.0
    xyz=[np.sin(p[:,1])*np.cos(p[:,0]),
         np.sin(p[:,1])*np.sin(p[:,0]),
         np.cos(p[:,1])]
    return np.array(xyz).T

# Convert from cartesion to spherical
def xyz2sph(p):
    lat=np.arccos(p[2])
    lon=np.arctan2(p[1],p[0])
    out=np.array([lon,lat])
    # Convert from rad to deg
    out*=180.0/np.pi
    out[1]-=90
    return out

# Get the angle in degrees between two vectors...
# ...assumes both vectors have a length of one (clips for rounding error in dot product)
# ...this is done to ignore speed reduction in repeatedly normalizing the station xyz vectors
def vecsAngle(v1,v2):
    return np.arccos(np.clip(np.dot(v1,v2),-1,1))*180.0/np.pi

# 3rd order polynomial fixed at 0,0
def poly3_Fixed(x,a,b,c):
    return np.clip(a*x**1+b*x**2+c*x**3,0,2000)

# 3rd order polynomial
def poly3(x,a,b,c,d):
    return np.clip(a*x**1+b*x**2+c*x**3+d,0,2000)

# Function to calculate the arrival times vs epicentral location
# ti=function((staX,staY,staZ,isPphase,isSphase,p1,p2,p3,s1,s2,s3),x0,y0,z0,t0)
# ti=t0+travelTime
def globalLocEpiFunc(data,x0,y0,z0,t0):
    eveXyz=np.array([x0,y0,z0])
    eveXyz=eveXyz/np.sqrt(np.sum(eveXyz**2)) # Normalize to length one
    paramP,paramS=data[:,5:8],data[:,8:11]
    degDists=vecsAngle(data[:,:3],eveXyz)
    ttP=poly3_Fixed(degDists,paramP[:,0],paramP[:,1],paramP[:,2])
    ttS=poly3_Fixed(degDists,paramS[:,0],paramS[:,1],paramS[:,2])
    return t0+data[:,3]*ttP+data[:,4]*ttS

# Function to calculate the arrival times vs depth
# ti=function((isPphase,isSphase,p1,p2,p3,p4,s1,s2,s3,s4),z0,t0)
# ti=t0+travelTime
def globalLocDepFunc(data,z0,t0):
    paramP,paramS=data[:,2:6],data[:,6:10]
    ttP=poly3(z0,paramP[:,0],paramP[:,1],paramP[:,2],paramP[:,3])
    ttS=poly3(z0,paramS[:,0],paramS[:,1],paramS[:,2],paramS[:,3])
    return t0+data[:,0]*ttP+data[:,1]*ttS

# Recalculate the current event-station distances with current event location
def recalcDegDists(data,params):
    # Normalize the params, as were not constrained in "curve_fit" to have magnitude of 1
    params[:3]=params[:3]/np.sqrt(np.sum(params[:3]**2))
    degDists=vecsAngle(data[:,:3],params[:3])
    return degDists,params

# Use travel times from the IASP91 model model to estimate the event location
# Gives approximate event location, less appropriate with smaller networks
# Depth is constrained from 0 to 200 km
# Method: A coarse search is done first using a third order polynomial fit, varying epicenter...
# ...a refined epicentral search is done using O(3) polynomials fits about the predicted station-event distances...
# ...a final search vs depth is done using O(3) polynomials at the refined station-event distances
# Special note: the scipy curve_fit function does not constrain variables, thus the length 1 vector...
# ...representing the surface position will not remain length 1; this effect is "ignored" as the...
# ...travel times are related to distance in degrees and so the orientation of this vector is all that matters
def globalLocate(pickSet,staLoc,mainPath,customDict,staProjStyle):
#    import time
#    now=time.time()
    # Nothing to do if no picks, or not in lon,lats
    if 0 in pickSet.shape:
        return np.empty((0,5)),customDict
    elif staProjStyle!='None':
        print('Global locate requires stations to use lon,lat')
        return np.empty((0,5)),customDict
    # Load in the model (if not already loaded into the custom dictionary)
    if 'globEpiDict' not in customDict.keys():
        with open(mainPath+'/Plugins/epiTTparams.pickle','rb') as aFile:
            customDict['globEpiDict']=pickle.load(aFile)
        with open(mainPath+'/Plugins/depTTparams.pickle','rb') as aFile:
            customDict['globDepDict']=pickle.load(aFile)
    ttEpiDict=customDict['globEpiDict']
    ttDepDict=customDict['globDepDict']
    # Remove anything but P and S picks
    for i,entry in enumerate(pickSet[:,1]):
        pickSet[i,1]=pickSet[i,1][0]
    pickSet=pickSet[np.where((pickSet[:,1]=='P')|(pickSet[:,1]=='S'))]
    # Get only picks which have station metadata
    pickSet,pickStaIdxs=getPickStaIdxs(pickSet,staLoc[:,0])
    
    # Generate the data for the rough global curve fitting...
    # ...(staX,staY,staZ,isPphase,isSphase,p1,p2,p3,s1,s2,s3)
    data=np.zeros((len(pickSet),11),dtype=float)
    # ...set the station locations
    data[:,:3]=sph2xyz(staLoc[pickStaIdxs,1:3].astype(float))
    # ...set phase markers
    data[:,3]=pickSet[:,1]=='P'
    data[:,4]=pickSet[:,1]=='S'
    # ...set the coefficients for the global travel time calculations
    data[:,5:]=np.concatenate((ttEpiDict['globP'],ttEpiDict['globS']))
    # ...get the "y" values to fit the curve to
    Tref=np.min(pickSet[:,2].astype(float))
    pickTimes=pickSet[:,2].astype(float)-Tref
    # Initial guess on origin, uses station location with earliest pick
    testLoc=data[np.argmin(pickTimes),:3] 
    testLoc=np.concatenate((testLoc,[0]))

    # Solve for event origin parameters
    try:
        params, pcov = curve_fit(globalLocEpiFunc,data,pickTimes,testLoc,
                                 bounds=([-1,-1,-1,-2000],[1,1,1,2000]))
    except:
        print('Global locator failed')
        return np.empty((0,5)),customDict
    degDists,params=recalcDegDists(data,params) # Normalize params
    
    # Update the data for the refined global curve fitting...
    # ...figure out which pre-fit parameters should be taken for each pick 
    # (could be by station but likely fast enough)
    bins=np.arange(0-0.5*ttEpiDict['spacing'],
                   180+0.51*ttEpiDict['spacing'],ttEpiDict['spacing'])
    idxs=np.digitize(degDists,bins)-1
    data[:,5:8]=ttEpiDict['pParams'][idxs]
    data[:,8:11]=ttEpiDict['sParams'][idxs]
    # Fit again with refined parameters, use previous location as starting position
    try:
        params, pcov = curve_fit(globalLocEpiFunc,data,pickTimes,params,
                                 bounds=([-1,-1,-1,-2000],[1,1,1,2000]))
    except:
        print('Global locator failed')
        return np.empty((0,5)),customDict
    # Convert back from the xyz to lon,lat
    degDists,params=recalcDegDists(data,params) # Normalize params
    lon,lat=xyz2sph(params[:3])
    
    # Generate the data for the depth curve fitting...
    bins=np.arange(0-0.5*ttDepDict['spacing'],
                   180+0.51*ttDepDict['spacing'],ttDepDict['spacing'])
    idxs=np.digitize(degDists,bins)-1
    dataDep=np.zeros((len(data),10),dtype=float)
    dataDep[:,:2]=data[:,3:5]
    dataDep[:,2:6]=ttDepDict['pParams'][idxs]
    dataDep[:,6:10]=ttDepDict['sParams'][idxs]
    # Fit again with depth parameters, use model starting depth and previous origin time as starting position
    try:
        depParams, pcov = curve_fit(globalLocDepFunc,dataDep,pickTimes,[ttEpiDict['startDep'],params[-1]],
                                 bounds=([0,-2000],[200,2000]))
    except:
        print('Global locator failed')
        return np.empty((0,5)),customDict
#    print time.time()-now,len(pickSet)
#    print lon,lat,depParams[0]
#    print(np.sum(np.abs(pickTimes-globalLocEpiFunc(data,*params)))/len(data),'preAvgResid')
#    print(np.sum(np.abs(pickTimes-globalLocDepFunc(dataDep,*depParams)))/len(data),'postAvgResig')
    return np.array([[0,lon,lat,depParams[0],depParams[1]+Tref]]),customDict



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ActionSetup.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_actionDialog(object):
    def setupUi(self, actionDialog):
        actionDialog.setObjectName("actionDialog")
        actionDialog.resize(626, 626)
        self.verticalLayout = QtWidgets.QVBoxLayout(actionDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.actShortInputLayout = QtWidgets.QGridLayout()
        self.actShortInputLayout.setSpacing(1)
        self.actShortInputLayout.setObjectName("actShortInputLayout")
        self.actOptionalsLabel = QtWidgets.QLabel(actionDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.actOptionalsLabel.sizePolicy().hasHeightForWidth())
        self.actOptionalsLabel.setSizePolicy(sizePolicy)
        self.actOptionalsLabel.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actOptionalsLabel.setFont(font)
        self.actOptionalsLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actOptionalsLabel.setObjectName("actOptionalsLabel")
        self.actShortInputLayout.addWidget(self.actOptionalsLabel, 6, 1, 1, 1)
        self.actTriggerLineEdit = KeyBindLineEdit(actionDialog)
        self.actTriggerLineEdit.setObjectName("actTriggerLineEdit")
        self.actShortInputLayout.addWidget(self.actTriggerLineEdit, 2, 4, 1, 1)
        self.actTagLabel = QtWidgets.QLabel(actionDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.actTagLabel.sizePolicy().hasHeightForWidth())
        self.actTagLabel.setSizePolicy(sizePolicy)
        self.actTagLabel.setBaseSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actTagLabel.setFont(font)
        self.actTagLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actTagLabel.setObjectName("actTagLabel")
        self.actShortInputLayout.addWidget(self.actTagLabel, 0, 1, 1, 1)
        self.actNameLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actNameLabel.setFont(font)
        self.actNameLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actNameLabel.setObjectName("actNameLabel")
        self.actShortInputLayout.addWidget(self.actNameLabel, 2, 1, 1, 1)
        self.actPassiveRadio = QtWidgets.QRadioButton(actionDialog)
        self.actPassiveRadio.setChecked(True)
        self.actPassiveRadio.setObjectName("actPassiveRadio")
        self.actShortInputLayout.addWidget(self.actPassiveRadio, 6, 3, 1, 1)
        self.actOptionalsLineEdit = QtWidgets.QLineEdit(actionDialog)
        self.actOptionalsLineEdit.setObjectName("actOptionalsLineEdit")
        self.actShortInputLayout.addWidget(self.actOptionalsLineEdit, 6, 2, 1, 1)
        self.actIntervalLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actIntervalLabel.setFont(font)
        self.actIntervalLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actIntervalLabel.setObjectName("actIntervalLabel")
        self.actShortInputLayout.addWidget(self.actIntervalLabel, 4, 3, 1, 1)
        self.actActiveRadio = QtWidgets.QRadioButton(actionDialog)
        self.actActiveRadio.setObjectName("actActiveRadio")
        self.actShortInputLayout.addWidget(self.actActiveRadio, 0, 3, 1, 1)
        self.actNameLineEdit = QtWidgets.QLineEdit(actionDialog)
        self.actNameLineEdit.setObjectName("actNameLineEdit")
        self.actShortInputLayout.addWidget(self.actNameLineEdit, 2, 2, 1, 1)
        self.passiveBeforeCheck = QtWidgets.QCheckBox(actionDialog)
        self.passiveBeforeCheck.setObjectName("passiveBeforeCheck")
        self.actShortInputLayout.addWidget(self.passiveBeforeCheck, 6, 4, 1, 1)
        self.actPathLabel = QtWidgets.QLabel(actionDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.actPathLabel.sizePolicy().hasHeightForWidth())
        self.actPathLabel.setSizePolicy(sizePolicy)
        self.actPathLabel.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actPathLabel.setFont(font)
        self.actPathLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actPathLabel.setObjectName("actPathLabel")
        self.actShortInputLayout.addWidget(self.actPathLabel, 4, 1, 1, 1)
        self.actTagLineEdit = HoverLineEdit(actionDialog)
        self.actTagLineEdit.setObjectName("actTagLineEdit")
        self.actShortInputLayout.addWidget(self.actTagLineEdit, 0, 2, 1, 1)
        self.actPathLineEdit = QtWidgets.QLineEdit(actionDialog)
        self.actPathLineEdit.setObjectName("actPathLineEdit")
        self.actShortInputLayout.addWidget(self.actPathLineEdit, 4, 2, 1, 1)
        self.actTriggerLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actTriggerLabel.setFont(font)
        self.actTriggerLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actTriggerLabel.setObjectName("actTriggerLabel")
        self.actShortInputLayout.addWidget(self.actTriggerLabel, 2, 3, 1, 1)
        self.actIntervalLineEdit = QtWidgets.QLineEdit(actionDialog)
        self.actIntervalLineEdit.setObjectName("actIntervalLineEdit")
        self.actShortInputLayout.addWidget(self.actIntervalLineEdit, 4, 4, 1, 1)
        self.actActiveCheckLayout = QtWidgets.QHBoxLayout()
        self.actActiveCheckLayout.setObjectName("actActiveCheckLayout")
        self.activeTimerCheck = QtWidgets.QCheckBox(actionDialog)
        self.activeTimerCheck.setObjectName("activeTimerCheck")
        self.actActiveCheckLayout.addWidget(self.activeTimerCheck)
        self.activeThreadedCheck = QtWidgets.QCheckBox(actionDialog)
        self.activeThreadedCheck.setObjectName("activeThreadedCheck")
        self.actActiveCheckLayout.addWidget(self.activeThreadedCheck)
        self.actShortInputLayout.addLayout(self.actActiveCheckLayout, 0, 4, 1, 1)
        self.actShortInputLayout.setColumnStretch(1, 1)
        self.actShortInputLayout.setColumnStretch(2, 7)
        self.actShortInputLayout.setColumnStretch(3, 1)
        self.actShortInputLayout.setColumnStretch(4, 1)
        self.verticalLayout.addLayout(self.actShortInputLayout)
        self.actReturnLayout = QtWidgets.QGridLayout()
        self.actReturnLayout.setSpacing(1)
        self.actReturnLayout.setObjectName("actReturnLayout")
        self.actSelectInputList = KeyListWidget(actionDialog)
        self.actSelectInputList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.actSelectInputList.setObjectName("actSelectInputList")
        self.actReturnLayout.addWidget(self.actSelectInputList, 1, 2, 1, 1)
        self.actAvailReturnList = QtWidgets.QListWidget(actionDialog)
        self.actAvailReturnList.setObjectName("actAvailReturnList")
        self.actReturnLayout.addWidget(self.actAvailReturnList, 2, 3, 1, 1)
        self.actAvailTriggerList = QtWidgets.QListWidget(actionDialog)
        self.actAvailTriggerList.setObjectName("actAvailTriggerList")
        self.actReturnLayout.addWidget(self.actAvailTriggerList, 2, 1, 1, 1)
        self.actInputLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actInputLabel.setFont(font)
        self.actInputLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actInputLabel.setObjectName("actInputLabel")
        self.actReturnLayout.addWidget(self.actInputLabel, 0, 2, 1, 1)
        self.actReturnLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actReturnLabel.setFont(font)
        self.actReturnLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actReturnLabel.setObjectName("actReturnLabel")
        self.actReturnLayout.addWidget(self.actReturnLabel, 0, 3, 1, 1)
        self.actSelectReturnList = KeyListWidget(actionDialog)
        self.actSelectReturnList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.actSelectReturnList.setObjectName("actSelectReturnList")
        self.actReturnLayout.addWidget(self.actSelectReturnList, 1, 3, 1, 1)
        self.actAvailInputList = QtWidgets.QListWidget(actionDialog)
        self.actAvailInputList.setViewMode(QtWidgets.QListView.ListMode)
        self.actAvailInputList.setObjectName("actAvailInputList")
        self.actReturnLayout.addWidget(self.actAvailInputList, 2, 2, 1, 1)
        self.actSelectLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actSelectLabel.setFont(font)
        self.actSelectLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actSelectLabel.setObjectName("actSelectLabel")
        self.actReturnLayout.addWidget(self.actSelectLabel, 1, 0, 1, 1)
        self.actSelectTriggerList = KeyListWidget(actionDialog)
        self.actSelectTriggerList.setObjectName("actSelectTriggerList")
        self.actReturnLayout.addWidget(self.actSelectTriggerList, 1, 1, 1, 1)
        self.actAvailLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actAvailLabel.setFont(font)
        self.actAvailLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actAvailLabel.setObjectName("actAvailLabel")
        self.actReturnLayout.addWidget(self.actAvailLabel, 2, 0, 1, 1)
        self.actPassiveTriggerLabel = QtWidgets.QLabel(actionDialog)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.actPassiveTriggerLabel.setFont(font)
        self.actPassiveTriggerLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.actPassiveTriggerLabel.setObjectName("actPassiveTriggerLabel")
        self.actReturnLayout.addWidget(self.actPassiveTriggerLabel, 0, 1, 1, 1)
        self.actReturnLayout.setRowStretch(1, 1)
        self.actReturnLayout.setRowStretch(2, 2)
        self.verticalLayout.addLayout(self.actReturnLayout)
        self.actLowerLayout = QtWidgets.QHBoxLayout()
        self.actLowerLayout.setObjectName("actLowerLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.actLowerLayout.addItem(spacerItem)
        self.actButtonBox = QtWidgets.QDialogButtonBox(actionDialog)
        self.actButtonBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.actButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.actButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.actButtonBox.setCenterButtons(True)
        self.actButtonBox.setObjectName("actButtonBox")
        self.actLowerLayout.addWidget(self.actButtonBox)
        self.actLowerLayout.setStretch(0, 20)
        self.actLowerLayout.setStretch(1, 1)
        self.verticalLayout.addLayout(self.actLowerLayout)

        self.retranslateUi(actionDialog)
        self.actButtonBox.accepted.connect(actionDialog.accept)
        self.actButtonBox.rejected.connect(actionDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(actionDialog)
        actionDialog.setTabOrder(self.actTagLineEdit, self.actNameLineEdit)
        actionDialog.setTabOrder(self.actNameLineEdit, self.actPathLineEdit)
        actionDialog.setTabOrder(self.actPathLineEdit, self.actOptionalsLineEdit)
        actionDialog.setTabOrder(self.actOptionalsLineEdit, self.actActiveRadio)
        actionDialog.setTabOrder(self.actActiveRadio, self.actTriggerLineEdit)
        actionDialog.setTabOrder(self.actTriggerLineEdit, self.actIntervalLineEdit)
        actionDialog.setTabOrder(self.actIntervalLineEdit, self.actPassiveRadio)
        actionDialog.setTabOrder(self.actPassiveRadio, self.passiveBeforeCheck)
        actionDialog.setTabOrder(self.passiveBeforeCheck, self.actSelectTriggerList)
        actionDialog.setTabOrder(self.actSelectTriggerList, self.actAvailTriggerList)
        actionDialog.setTabOrder(self.actAvailTriggerList, self.actSelectInputList)
        actionDialog.setTabOrder(self.actSelectInputList, self.actAvailInputList)
        actionDialog.setTabOrder(self.actAvailInputList, self.actSelectReturnList)
        actionDialog.setTabOrder(self.actSelectReturnList, self.actAvailReturnList)
        actionDialog.setTabOrder(self.actAvailReturnList, self.actButtonBox)

    def retranslateUi(self, actionDialog):
        _translate = QtCore.QCoreApplication.translate
        actionDialog.setWindowTitle(_translate("actionDialog", "Setup Action"))
        self.actOptionalsLabel.setToolTip(_translate("actionDialog", "Optionals to send to function"))
        self.actOptionalsLabel.setText(_translate("actionDialog", "Optionals"))
        self.actTagLabel.setToolTip(_translate("actionDialog", "Human readable name of action"))
        self.actTagLabel.setText(_translate("actionDialog", "Tag "))
        self.actNameLabel.setToolTip(_translate("actionDialog", "Function name of action"))
        self.actNameLabel.setText(_translate("actionDialog", "Name"))
        self.actPassiveRadio.setToolTip(_translate("actionDialog", "Action is triggered by an active action"))
        self.actPassiveRadio.setText(_translate("actionDialog", "Passive"))
        self.actIntervalLabel.setToolTip(_translate("actionDialog", "Time in seconds between calling action"))
        self.actIntervalLabel.setText(_translate("actionDialog", "Interval (s)"))
        self.actActiveRadio.setToolTip(_translate("actionDialog", "Action is called using a keybind or click"))
        self.actActiveRadio.setText(_translate("actionDialog", "Active"))
        self.passiveBeforeCheck.setToolTip(_translate("actionDialog", "Action is to applied prior its triggering active action"))
        self.passiveBeforeCheck.setText(_translate("actionDialog", "Before Trigger"))
        self.actPathLabel.setToolTip(_translate("actionDialog", "Path relative a directory on the computers PATH variable"))
        self.actPathLabel.setText(_translate("actionDialog", "Path"))
        self.actTriggerLabel.setToolTip(_translate("actionDialog", "Keybind to activate action"))
        self.actTriggerLabel.setText(_translate("actionDialog", "Trigger"))
        self.activeTimerCheck.setToolTip(_translate("actionDialog", "Repeat action every interval"))
        self.activeTimerCheck.setText(_translate("actionDialog", "Timer"))
        self.activeThreadedCheck.setToolTip(_translate("actionDialog", "Run action in background"))
        self.activeThreadedCheck.setText(_translate("actionDialog", "Thread"))
        self.actAvailReturnList.setSortingEnabled(True)
        self.actAvailTriggerList.setSortingEnabled(True)
        self.actInputLabel.setToolTip(_translate("actionDialog", "Variable(s) to send to function"))
        self.actInputLabel.setText(_translate("actionDialog", "Inputs"))
        self.actReturnLabel.setToolTip(_translate("actionDialog", "Variable(s) to be returned by function "))
        self.actReturnLabel.setText(_translate("actionDialog", "Returns"))
        self.actAvailInputList.setSortingEnabled(True)
        self.actSelectLabel.setText(_translate("actionDialog", "Selected"))
        self.actSelectTriggerList.setSortingEnabled(True)
        self.actAvailLabel.setText(_translate("actionDialog", "Available"))
        self.actPassiveTriggerLabel.setToolTip(_translate("actionDialog", "Active actions which trigger a passive action"))
        self.actPassiveTriggerLabel.setText(_translate("actionDialog", "Passive Triggers"))

from CustomWidgets import HoverLineEdit, KeyBindLineEdit, KeyListWidget



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ChangeSource.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CsDialog(object):
    def setupUi(self, CsDialog):
        CsDialog.setObjectName("CsDialog")
        CsDialog.resize(646, 529)
        CsDialog.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.verticalLayout = QtWidgets.QVBoxLayout(CsDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.csLineEditLayout = QtWidgets.QGridLayout()
        self.csLineEditLayout.setSpacing(1)
        self.csLineEditLayout.setObjectName("csLineEditLayout")
        self.csStationLabel = QtWidgets.QLabel(CsDialog)
        self.csStationLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.csStationLabel.setObjectName("csStationLabel")
        self.csLineEditLayout.addWidget(self.csStationLabel, 3, 0, 1, 1)
        self.csPickLabel = QtWidgets.QLabel(CsDialog)
        self.csPickLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.csPickLabel.setObjectName("csPickLabel")
        self.csLineEditLayout.addWidget(self.csPickLabel, 2, 0, 1, 1)
        self.csStationLineEdit = DblClickLineEdit(CsDialog)
        self.csStationLineEdit.setObjectName("csStationLineEdit")
        self.csLineEditLayout.addWidget(self.csStationLineEdit, 3, 1, 1, 1)
        self.csTagLineEdit = QtWidgets.QLineEdit(CsDialog)
        self.csTagLineEdit.setObjectName("csTagLineEdit")
        self.csLineEditLayout.addWidget(self.csTagLineEdit, 0, 1, 1, 1)
        self.csPickLineEdit = DblClickLineEdit(CsDialog)
        self.csPickLineEdit.setObjectName("csPickLineEdit")
        self.csLineEditLayout.addWidget(self.csPickLineEdit, 2, 1, 1, 1)
        self.csTagLabel = QtWidgets.QLabel(CsDialog)
        self.csTagLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.csTagLabel.setObjectName("csTagLabel")
        self.csLineEditLayout.addWidget(self.csTagLabel, 0, 0, 1, 1)
        self.csArchiveLabel = QtWidgets.QLabel(CsDialog)
        self.csArchiveLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.csArchiveLabel.setObjectName("csArchiveLabel")
        self.csLineEditLayout.addWidget(self.csArchiveLabel, 1, 0, 1, 1)
        self.csArchiveLineEdit = DblClickLineEdit(CsDialog)
        self.csArchiveLineEdit.setObjectName("csArchiveLineEdit")
        self.csLineEditLayout.addWidget(self.csArchiveLineEdit, 1, 1, 1, 1)
        self.csSaveSourceButton = QtWidgets.QPushButton(CsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.csSaveSourceButton.sizePolicy().hasHeightForWidth())
        self.csSaveSourceButton.setSizePolicy(sizePolicy)
        self.csSaveSourceButton.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.csSaveSourceButton.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.csSaveSourceButton.setObjectName("csSaveSourceButton")
        self.csLineEditLayout.addWidget(self.csSaveSourceButton, 3, 2, 1, 1)
        self.verticalLayout.addLayout(self.csLineEditLayout)
        self.csLowerLayout = QtWidgets.QGridLayout()
        self.csLowerLayout.setSpacing(1)
        self.csLowerLayout.setObjectName("csLowerLayout")
        self.csSaveSourceLabel = QtWidgets.QLabel(CsDialog)
        self.csSaveSourceLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.csSaveSourceLabel.setObjectName("csSaveSourceLabel")
        self.csLowerLayout.addWidget(self.csSaveSourceLabel, 1, 0, 1, 1)
        self.csSaveSourceList = KeyListWidget(CsDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.csSaveSourceList.sizePolicy().hasHeightForWidth())
        self.csSaveSourceList.setSizePolicy(sizePolicy)
        self.csSaveSourceList.setMinimumSize(QtCore.QSize(130, 0))
        self.csSaveSourceList.setObjectName("csSaveSourceList")
        self.csLowerLayout.addWidget(self.csSaveSourceList, 2, 0, 1, 1)
        self.csSourceDictLabel = QtWidgets.QLabel(CsDialog)
        self.csSourceDictLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.csSourceDictLabel.setObjectName("csSourceDictLabel")
        self.csLowerLayout.addWidget(self.csSourceDictLabel, 1, 1, 1, 1)
        self.sourceDictWidget = SourceDictWidget(CsDialog)
        self.sourceDictWidget.setObjectName("sourceDictWidget")
        self.csLowerLayout.addWidget(self.sourceDictWidget, 2, 1, 1, 1)
        self.csLowerLayout.setColumnStretch(0, 1)
        self.csLowerLayout.setColumnStretch(1, 4)
        self.verticalLayout.addLayout(self.csLowerLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(CsDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(CsDialog)
        self.buttonBox.accepted.connect(CsDialog.accept)
        self.buttonBox.rejected.connect(CsDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(CsDialog)
        CsDialog.setTabOrder(self.csTagLineEdit, self.csArchiveLineEdit)
        CsDialog.setTabOrder(self.csArchiveLineEdit, self.csPickLineEdit)
        CsDialog.setTabOrder(self.csPickLineEdit, self.csStationLineEdit)
        CsDialog.setTabOrder(self.csStationLineEdit, self.csSaveSourceButton)
        CsDialog.setTabOrder(self.csSaveSourceButton, self.csSaveSourceList)
        CsDialog.setTabOrder(self.csSaveSourceList, self.buttonBox)

    def retranslateUi(self, CsDialog):
        _translate = QtCore.QCoreApplication.translate
        CsDialog.setWindowTitle(_translate("CsDialog", "Change Source"))
        self.csStationLabel.setText(_translate("CsDialog", "Station File"))
        self.csPickLabel.setText(_translate("CsDialog", "Pick Directory"))
        self.csTagLabel.setText(_translate("CsDialog", "Tag"))
        self.csArchiveLabel.setText(_translate("CsDialog", "Archive Directory"))
        self.csSaveSourceButton.setText(_translate("CsDialog", "Save Source"))
        self.csSaveSourceLabel.setText(_translate("CsDialog", "Saved Sources"))
        self.csSourceDictLabel.setText(_translate("CsDialog", "Source Dictionary"))

from CustomWidgets import DblClickLineEdit, KeyListWidget, SourceDictWidget
